@@ -109,14 +109,14 @@ def list_installed(installdata: backends.InstallData) -> T.Dict[str, str]:
             for alias in t.aliases.keys():
                 res[os.path.join(installdata.build_dir, alias)] = \
                     os.path.join(installdata.prefix, t.outdir, os.path.basename(alias))
-        for path, installpath, _ in installdata.data:
-            res[path] = os.path.join(installdata.prefix, installpath)
-        for path, installdir, _ in installdata.headers:
-            res[path] = os.path.join(installdata.prefix, installdir, os.path.basename(path))
-        for path, installpath, _ in installdata.man:
-            res[path] = os.path.join(installdata.prefix, installpath)
-        for path, installpath, _, _ in installdata.install_subdirs:
-            res[path] = os.path.join(installdata.prefix, installpath)
+        for i in installdata.data:
+            res[i.path] = os.path.join(installdata.prefix, i.install_path)
+        for i in installdata.headers:
+            res[i.path] = os.path.join(installdata.prefix, i.install_path, os.path.basename(i.path))
+        for i in installdata.man:
+            res[i.path] = os.path.join(installdata.prefix, i.install_path)
+        for i in installdata.install_subdirs:
+            res[i.path] = os.path.join(installdata.prefix, i.install_path)
     return res
 
 def get_target_dir(coredata: cdata.CoreData, subdir: str) -> str:
@@ -391,10 +391,10 @@ def get_infodir(builddir: T.Optional[str] = None) -> str:
 
 def get_info_file(infodir: str, kind: T.Optional[str] = None) -> str:
     return os.path.join(infodir,
-                        'meson-info.json' if not kind else 'intro-{}.json'.format(kind))
+                        'meson-info.json' if not kind else f'intro-{kind}.json')
 
 def load_info_file(infodir: str, kind: T.Optional[str] = None) -> T.Any:
-    with open(get_info_file(infodir, kind), 'r') as fp:
+    with open(get_info_file(infodir, kind)) as fp:
         return json.load(fp)
 
 def run(options: argparse.Namespace) -> int:
@@ -509,7 +509,7 @@ def write_meson_info_file(builddata: build.Build, errors: list, build_files_upda
         if not intro_types[i].func:
             continue
         intro_info[i] = {
-            'file': 'intro-{}.json'.format(i),
+            'file': f'intro-{i}.json',
             'updated': i in updated_introspection_files
         }
 
