@@ -14,7 +14,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""This script reads config.h.meson, looks for header
+help_message = """Usage: {} <config.h.meson>
+
+This script reads config.h.meson, looks for header
 checks and writes the corresponding meson declaration.
 
 Copy config.h.in to config.h.meson, replace #undef
@@ -39,7 +41,6 @@ function_data = \
      'HAVE_POSIX_MEMALIGN': ('posix_memalign', 'stdlib.h'),
      'HAVE_SIGACTION': ('sigaction', 'signal.h'),
      'HAVE_ALARM': ('alarm', 'unistd.h'),
-     'HAVE_CLOCK_GETTIME': ('clock_gettime', 'time.h'),
      'HAVE_CTIME_R': ('ctime_r', 'time.h'),
      'HAVE_DRAND48': ('drand48', 'stdlib.h'),
      'HAVE_FLOCKFILE': ('flockfile', 'stdio.h'),
@@ -365,6 +366,11 @@ function_data = \
 headers = []
 functions = []
 sizes = []
+
+if len(sys.argv) != 2:
+    print(help_message.format(sys.argv[0]))
+    sys.exit(0)
+
 with open(sys.argv[1]) as f:
     for line in f:
         line = line.strip()
@@ -383,9 +389,9 @@ with open(sys.argv[1]) as f:
             token = arr[1]
             if token in function_data:
                 fdata = function_data[token]
-                functions.append((token, fdata[0], fdata[1]))
+                functions.append([token, fdata[0], fdata[1]])
             elif token.startswith('HAVE_') and not token.endswith('_H'):
-                functions.append((token, ))
+                functions.append([token])
         except Exception:
             pass
 
@@ -408,7 +414,7 @@ cdata = configuration_data()''')
 
 print('check_headers = [')
 for token, hname in headers:
-    print("  ['%s', '%s']," % (token, hname))
+    print(f"  ['{token}', '{hname}'],")
 print(']\n')
 
 print('''foreach h : check_headers
@@ -421,12 +427,12 @@ endforeach
 # Convert function checks.
 
 print('check_functions = [')
-for token in functions:
-    if len(token) == 3:
-        token, fdata0, fdata1 = token
-        print("  ['%s', '%s', '#include<%s>']," % (token, fdata0, fdata1))
+for tok in functions:
+    if len(tok) == 3:
+        tokstr, fdata0, fdata1 = tok
+        print(f"  ['{tokstr}', '{fdata0}', '#include<{fdata1}>'],")
     else:
-        print('# check token', token)
+        print('# check token', tok)
 print(']\n')
 
 print('''foreach f : check_functions
@@ -439,7 +445,7 @@ endforeach
 # Convert sizeof checks.
 
 for elem, typename in sizes:
-    print("cdata.set('%s', cc.sizeof('%s'))" % (elem, typename))
+    print(f"cdata.set('{elem}', cc.sizeof('{typename}'))")
 
 print('''
 configure_file(input : 'config.h.meson',
